// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
			 "cloudera/chart/spec/MockAnyChartStock",
			 "cloudera/chart/AnyStockTimeBrowser",
			 "cloudera/chart/TimeBrowserState"
], function(MockAnyChartStock, AnyStockTimeBrowser, TimeBrowserState) {

	describe("AnyStockTimeBrowser Tests", function() {
		var timeBrowser;

		beforeEach(function() {
			var options = {
				firstDate: new Date(910),
				lastDate: new Date(1000),
				firstVisibleDate: new Date(950),
				lastVisibleDate: new Date(1000),
				markerDate: new Date(1000),
				isCurrentMode: true,
				timeSeriesDescriptors : [ {
					"context" : {
						"id": "cluster",
						"type": "CLUSTER"
					},
					"metric" : {
						"id" : 3,
						"context" : "CLUSTER",
						"name" : "maps_running",
						"desc" : "Running map tasks",
						"title" : "Running maps",
						"selector" : "AVERAGE"
					},
					"chart" : {
						"id" : "CLUSTER_3",
						"title" : "Running maps",
						"discrete" : true
					}
				} ],
				mode: "INTERACTIVE"
			};
			var state = new TimeBrowserState(options);
			var options2 = $.extend({state: state}, options);
			timeBrowser = new AnyStockTimeBrowser(options2, "testId");
		});

		afterEach(function() {
			if (timeBrowser) {
				timeBrowser.unsubscribe();
			}
		});

		describe("when the timeBrowser is created", function() {
			var config;
			beforeEach(function() {
				config = timeBrowser.getConfig();
			});

			it("should see 10 generalDataProviders.", function() {
				// We expect the time control to have configuration to have the following parameters.
				expect(config.data.dataProviders.generalDataProviders.length).toEqual(10);
				expect(config.data.dataProviders.generalDataProviders[0].dataSet).toEqual("dataSet_0");
				expect(config.data.dataProviders.generalDataProviders[0].id).toEqual("dp_0");
			});

			it("should see 1 scrollerDataProviders.", function() {
				var config = timeBrowser.getConfig();

				expect(config.data.dataProviders.scrollerDataProviders.length).toEqual(1);
				expect(config.data.dataProviders.scrollerDataProviders[0].dataSet).toEqual("dataSet_0");
				expect(config.data.dataProviders.scrollerDataProviders[0].id).toEqual("dp_0");
			});

			it("should see 1 dataSets.", function() {
				var config = timeBrowser.getConfig();

				expect(config.data.dataSets.length).toEqual(10);
				expect(config.data.dataSets[0].id).toEqual("dataSet_0");
			});

			it("should see 1 chart.", function() {
				var config = timeBrowser.getConfig();

				expect(config.settings.charts.length).toEqual(1);
				expect(config.settings.charts[0].id).toEqual("CLUSTER_3");
				expect(config.settings.charts[0].seriesList.length).toEqual(1);
				expect(config.settings.charts[0].seriesList[0].dataProvider).toEqual("dp_0");
				expect(config.settings.charts[0].seriesList[0].id).toEqual("series_CLUSTER_cluster_3");
				expect(config.settings.charts[0].seriesList[0].name).toEqual("Running maps");
				expect(config.settings.charts[0].seriesList[0].type).toEqual("Line");
			});
		});
	});
});
